#import <React/RCTBridge.h>
#import <React/RCTUIManager.h>
#import <React/RCTConvert.h>
#import <React/RCTBridgeModule.h>
#import <React/RCTEventDispatcher.h>
#import <React/UIView+React.h>
#include <MediaAccessibility/MediaAccessibility.h>
#include <AVFoundation/AVFoundation.h>

#import "RNPrismPlayer.h"

@implementation RNPrismPlayer:UIView {
    RCTEventDispatcher *_eventDispatcher;
    NSDictionary *videoSoure;
    AVPlayer *player;
    AVPlayerLayer *videoLayer;
}
-(instancetype) initWithEventDispatcher:(RCTEventDispatcher *)eventDispatcher
{
    if(self == [super init]){
        _eventDispatcher = eventDispatcher;
    }
    
    return self;
}

-(void)setSource:(NSDictionary *)source
{
    player = [AVPlayer playerWithURL:[NSURL URLWithString:[source valueForKey:@"uri"]]];
    player.actionAtItemEnd = AVPlayerActionAtItemEndNone;
    videoLayer = [AVPlayerLayer playerLayerWithPlayer:player];
    videoLayer.frame = CGRectMake(0, 0, [UIScreen mainScreen].bounds.size.width, [UIScreen mainScreen].bounds.size.width * 9 / 16);
    videoLayer.videoGravity = AVLayerVideoGravityResizeAspectFill;
}


// React Methonds

-(void)layoutSubviews{
    [super layoutSubviews];
    [self.layer addSublayer:videoLayer];
    [player play];
}


/*
    Helper methods
 */
-(NSString *)convertToString:(NSDictionary *) dictionary{
    //* Converts Source dictionary to json String
    NSError * err;
    NSData *gSoruce = [NSJSONSerialization dataWithJSONObject:dictionary options:0 error:&err];
    NSString *sourceStr = [[NSString alloc] initWithData:gSoruce encoding:NSUTF8StringEncoding];
    return sourceStr;
}

-(Float64 *)getCurrentTime:(Float64 *)currentTime{
    return currentTime;
}
- (NSString *)timeFormatted:(int)totalSeconds
{
    int seconds = totalSeconds % 60;
    int minutes = (totalSeconds / 60) % 60;
    int hours = totalSeconds / 3600;
    return [NSString stringWithFormat:@"%02d:%02d:%02d",hours, minutes, seconds];
}
- (void)listSubviewsOfView:(UIView *)view {
    // Get the subviews of the view
    NSArray *subviews = [view subviews];
    // Return if there are no subviews
    if ([subviews count] == 0) return; // COUNT CHECK LINE
    for (UIView *subview in subviews) {
        // Do what you want to do with the subview
        // List the subviews of subview
        [self listSubviewsOfView:subview];
    }
}
- (UIImage *)imageWithColor:(UIColor *)color {
    CGRect rect = CGRectMake(0.0f, 0.0f, 1.0f, 1.0f);
    UIGraphicsBeginImageContext(rect.size);
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetFillColorWithColor(context, [color CGColor]);
    CGContextFillRect(context, rect);
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return image;
}

/******************************************************************************************/


@end

@implementation RNPrismPlayerManager

RCT_EXPORT_MODULE();

- (UIView *)view
{
    return [[RNPrismPlayer alloc] initWithEventDispatcher:self.bridge.eventDispatcher];
}

- (dispatch_queue_t)methodQueue
{
    return self.bridge.uiManager.methodQueue;
}

RCT_EXPORT_VIEW_PROPERTY(source, NSDictionary);

@end
